
import java.io.*;


import org.apache.bcel.*;
import org.apache.bcel.generic.*;
import org.apache.bcel.classfile.*;
import org.apache.bcel.util.*;



public class ShowClass {


	public static void writeConstantPool(JavaClass clazz) {

		System.out.println(clazz.getConstantPool());

	}

	public static void writeFields(JavaClass clazz) {

		Field [] fields = clazz.getFields();

		for(int i = 0; i < fields.length; ++i) {
			System.out.println(fields[i]);
		}

	}

	public static void writeMethods(JavaClass clazz) {

		ClassGen           clazzGenerator        = new ClassGen(clazz);
		ConstantPoolGen    constantPoolGenerator = clazzGenerator.getConstantPool();
		ConstantPool 	   cp		 	 = constantPoolGenerator.getConstantPool();

		Method [] methods = clazzGenerator.getMethods();

		for(int i = 0; i < methods.length; ++i) {
			System.out.println(methods[i]);
			MethodGen mg = new MethodGen(methods[i],clazzGenerator.getClassName(), constantPoolGenerator);
			System.out.println("maxlocals = " + 
					mg.getMaxLocals() +
					" maxstack = " + 
					mg.getMaxStack());
			InstructionList instrList = mg.getInstructionList();
			if(instrList != null) {
				InstructionHandle [] instructionHandles = instrList.getInstructionHandles();
				for(int j = 0; j < instructionHandles.length; ++j) {
					System.out.print(instructionHandles[j].getPosition() + ":\t");
					System.out.println(instructionHandles[j].getInstruction().toString(cp));
				}
				//		    System.out.println(instrList.toString(true));
			}
			else {
				System.out.println("NATIVE METHOD");
			}

			System.out.println("\n\n");

		}

	}


	public static void writeInterfaces(JavaClass clazz) {

		ClassGen           clazzGenerator        = new ClassGen(clazz);

		String [] interfaces = clazzGenerator.getInterfaceNames();
		for(int i = 0; i < interfaces.length; ++i) {
			System.out.println("interface -> " + interfaces[i]);
		}

	}


	public static void main(String [] args) throws IOException {

		if(args.length == 0) {
			System.err.println("You must give a valid filename.");
			System.exit(-1);
		}

		File file = new File(args[0]);

		JavaClass clazz;
		if(! (file.exists() && file.isFile()) ) {
			clazz = Repository.lookupClass(args[0]);
		}
		else {
			ClassParser cParser = new ClassParser(args[0]);
			clazz = cParser.parse();
		}
		if(clazz == null) {
			System.exit(42);
		}

		System.out.println("constants ---->");
		writeConstantPool(clazz);
		System.out.println("fields ------->");
		writeFields(clazz);
		System.out.println("interfaces --->");
		writeInterfaces(clazz);
		System.out.println("methods ------>");
		writeMethods(clazz);

	}


}
